@@ -75,9 +75,7 @@ class MOZ_STACK_CLASS nsHtml5OtherDocUpdate {
 };
 
 nsHtml5TreeOperation::nsHtml5TreeOperation()
-#ifdef DEBUG
  : mOpCode(eTreeOpUninitialized)
-#endif
 {
   MOZ_COUNT_CTOR(nsHtml5TreeOperation);
 }
@@ -102,6 +100,7 @@ nsHtml5TreeOperation::~nsHtml5TreeOperation()
     case eTreeOpAppendComment:
     case eTreeOpAppendCommentToDocument:
     case eTreeOpAddViewSourceHref:
+    case eTreeOpAddViewSourceBase:
       delete[] mTwo.unicharPtr;
       break;
     case eTreeOpSetDocumentCharset:
@@ -109,7 +108,7 @@ nsHtml5TreeOperation::~nsHtml5TreeOperation()
       delete[] mOne.charPtr;
       break;
     case eTreeOpProcessOfflineManifest:
-      nsMemory::Free(mOne.unicharPtr);
+      free(mOne.unicharPtr);
       break;
     default: // keep the compiler happy
       break;
@@ -159,7 +158,8 @@ nsHtml5TreeOperation::AppendText(const char16_t* aBuffer,
                                 aBuilder);
   }
 
-  nsRefPtr<nsTextNode> text = new nsTextNode(aBuilder->GetNodeInfoManager());
+  nsNodeInfoManager* nodeInfoManager = aParent->OwnerDoc()->NodeInfoManager();
+  RefPtr<nsTextNode> text = new nsTextNode(nodeInfoManager);
   NS_ASSERTION(text, "Infallible malloc failed?");
   rv = text->SetText(aBuffer, aLength, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -198,6 +198,10 @@ nsHtml5TreeOperation::AppendToDocument(nsIContent* aNode,
   nsIDocument* doc = aBuilder->GetDocument();
   uint32_t childCount = doc->GetChildCount();
   rv = doc->AppendChildTo(aNode, false);
+  if (rv == NS_ERROR_DOM_HIERARCHY_REQUEST_ERR) {
+    aNode->SetParserHasNotified();
+    return NS_OK;
+  }
   NS_ENSURE_SUCCESS(rv, rv);
   aNode->SetParserHasNotified();
   nsNodeUtils::ContentInserted(doc, aNode, childCount);
@@ -332,6 +336,7 @@ nsHtml5TreeOperation::CreateElement(int32_t aNs,
                                     nsIAtom* aName,
                                     nsHtml5HtmlAttributes* aAttributes,
                                     mozilla::dom::FromParser aFromParser,
+                                    nsNodeInfoManager* aNodeInfoManager,
                                     nsHtml5DocumentBuilder* aBuilder)
 {
   bool isKeygen = (aName == nsHtml5Atoms::keygen && aNs == kNameSpaceID_XHTML);
@@ -340,7 +345,7 @@ nsHtml5TreeOperation::CreateElement(int32_t aNs,
   }
 
   nsCOMPtr<dom::Element> newElement;
-  nsRefPtr<dom::NodeInfo> nodeInfo = aBuilder->GetNodeInfoManager()->
+  RefPtr<dom::NodeInfo> nodeInfo = aNodeInfoManager->
     GetNodeInfo(aName, nullptr, aNs, nsIDOMNode::ELEMENT_NODE);
   NS_ASSERTION(nodeInfo, "Got null nodeinfo.");
   NS_NewElement(getter_AddRefs(newElement),
@@ -379,20 +384,19 @@ nsHtml5TreeOperation::CreateElement(int32_t aNs,
                         theAttribute,
                         false);
 
-    nsRefPtr<dom::NodeInfo> optionNodeInfo =
-      aBuilder->GetNodeInfoManager()->GetNodeInfo(nsHtml5Atoms::option,
-                                                  nullptr,
-                                                  kNameSpaceID_XHTML,
-                                                  nsIDOMNode::ELEMENT_NODE);
+    RefPtr<dom::NodeInfo> optionNodeInfo =
+      aNodeInfoManager->GetNodeInfo(nsHtml5Atoms::option,
+                                    nullptr,
+                                    kNameSpaceID_XHTML,
+                                    nsIDOMNode::ELEMENT_NODE);
 
     for (uint32_t i = 0; i < theContent.Length(); ++i) {
       nsCOMPtr<dom::Element> optionElt;
-      nsRefPtr<dom::NodeInfo> ni = optionNodeInfo;
+      RefPtr<dom::NodeInfo> ni = optionNodeInfo;
       NS_NewElement(getter_AddRefs(optionElt),
                     ni.forget(),
                     aFromParser);
-      nsRefPtr<nsTextNode> optionText =
-        new nsTextNode(aBuilder->GetNodeInfoManager());
+      RefPtr<nsTextNode> optionText = new nsTextNode(aNodeInfoManager);
       (void) optionText->SetText(theContent[i], false);
       optionElt->AppendChildTo(optionText, false);
       newContent->AppendChildTo(optionElt, false);
@@ -406,8 +410,7 @@ nsHtml5TreeOperation::CreateElement(int32_t aNs,
   }
 
   int32_t len = aAttributes->getLength();
-  for (int32_t i = len; i > 0;) {
-    --i;
+  for (int32_t i = 0; i < len; i++) {
     // prefix doesn't need regetting. it is always null or a static atom
     // local name is never null
     nsCOMPtr<nsIAtom> localName =
@@ -435,14 +438,11 @@ nsHtml5TreeOperation::CreateElement(int32_t aNs,
                           value,
                           false);
 
-      // Custom element prototype swizzling may be needed if there is an
-      // "is" attribute.
+      // Custom element setup may be needed if there is an "is" attribute.
       if (kNameSpaceID_None == nsuri && !prefix && nsGkAtoms::is == localName) {
-        ErrorResult errorResult;
-        newContent->OwnerDoc()->SwizzleCustomElement(newContent,
-                                                     value,
-                                                     newContent->GetNameSpaceID(),
-                                                     errorResult);
+        newContent->OwnerDoc()->SetupCustomElement(newContent,
+                                                   newContent->GetNameSpaceID(),
+                                                   &value);
       }
     }
   }
@@ -463,7 +463,7 @@ nsHtml5TreeOperation::SetFormElement(nsIContent* aNode, nsIContent* aParent)
       !aNode->HasAttr(kNameSpaceID_None, nsGkAtoms::form)) {
     formControl->SetForm(formElement);
   } else if (domImageElement) {
-    nsRefPtr<dom::HTMLImageElement> imageElement =
+    RefPtr<dom::HTMLImageElement> imageElement =
       static_cast<dom::HTMLImageElement*>(domImageElement.get());
     MOZ_ASSERT(imageElement);
     imageElement->SetForm(formElement);
@@ -514,8 +514,8 @@ nsHtml5TreeOperation::FosterParentText(nsIContent* aStackParent,
                                   aBuilder);
     }
 
-    nsRefPtr<nsTextNode> text =
-      new nsTextNode(aBuilder->GetNodeInfoManager());
+    nsNodeInfoManager* nodeInfoManager = aStackParent->OwnerDoc()->NodeInfoManager();
+    RefPtr<nsTextNode> text = new nsTextNode(nodeInfoManager);
     NS_ASSERTION(text, "Infallible malloc failed?");
     rv = text->SetText(aBuffer, aLength, false);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -535,8 +535,8 @@ nsHtml5TreeOperation::AppendComment(nsIContent* aParent,
                                     int32_t aLength,
                                     nsHtml5DocumentBuilder* aBuilder)
 {
-  nsRefPtr<dom::Comment> comment =
-    new dom::Comment(aBuilder->GetNodeInfoManager());
+  nsNodeInfoManager* nodeInfoManager = aParent->OwnerDoc()->NodeInfoManager();
+  RefPtr<dom::Comment> comment = new dom::Comment(nodeInfoManager);
   NS_ASSERTION(comment, "Infallible malloc failed?");
   nsresult rv = comment->SetText(aBuffer, aLength, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -549,7 +549,7 @@ nsHtml5TreeOperation::AppendCommentToDocument(char16_t* aBuffer,
                                               int32_t aLength,
                                               nsHtml5DocumentBuilder* aBuilder)
 {
-  nsRefPtr<dom::Comment> comment =
+  RefPtr<dom::Comment> comment =
     new dom::Comment(aBuilder->GetNodeInfoManager());
   NS_ASSERTION(comment, "Infallible malloc failed?");
   nsresult rv = comment->SetText(aBuffer, aLength, false);
@@ -585,10 +585,17 @@ nsHtml5TreeOperation::GetDocumentFragmentForTemplate(nsIContent* aNode)
 {
   dom::HTMLTemplateElement* tempElem =
     static_cast<dom::HTMLTemplateElement*>(aNode);
-  nsRefPtr<dom::DocumentFragment> frag = tempElem->Content();
+  RefPtr<dom::DocumentFragment> frag = tempElem->Content();
   return frag;
 }
 
+nsIContent*
+nsHtml5TreeOperation::GetFosterParent(nsIContent* aTable, nsIContent* aStackParent)
+{
+  nsIContent* tableParent = aTable->GetParent();
+  return IsElementOrTemplateContent(tableParent) ? tableParent : aStackParent;
+}
+
 void
 nsHtml5TreeOperation::PreventScriptExecution(nsIContent* aNode)
 {
@@ -633,6 +640,9 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
                               nsIContent** aScriptElement)
 {
   switch(mOpCode) {
+    case eTreeOpUninitialized: {
+      MOZ_CRASH("eTreeOpUninitialized");
+    }
     case eTreeOpAppend: {
       nsIContent* node = *(mOne.node);
       nsIContent* parent = *(mTwo.node);
@@ -663,12 +673,23 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       nsHtml5HtmlAttributes* attributes = mTwo.attributes;
       return AddAttributes(node, attributes, aBuilder);
     }
+    case eTreeOpDocumentMode: {
+      aBuilder->SetDocumentMode(mOne.mode);
+      return NS_OK;
+    }
     case eTreeOpCreateElementNetwork:
     case eTreeOpCreateElementNotNetwork: {
       nsIContent** target = mOne.node;
       int32_t ns = mFour.integer;
       nsCOMPtr<nsIAtom> name = Reget(mTwo.atom);
       nsHtml5HtmlAttributes* attributes = mThree.attributes;
+      nsIContent* intendedParent = mFive.node ? *(mFive.node) : nullptr;
+
+      // intendedParent == nullptr is a special case where the
+      // intended parent is the document.
+      nsNodeInfoManager* nodeInfoManager = intendedParent ?
+         intendedParent->OwnerDoc()->NodeInfoManager() :
+         aBuilder->GetNodeInfoManager();
 
       *target = CreateElement(ns,
                               name,
@@ -676,6 +697,7 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
                               mOpCode == eTreeOpCreateElementNetwork ?
                                 dom::FROM_PARSER_NETWORK :
                                 dom::FROM_PARSER_DOCUMENT_WRITE,
+                              nodeInfoManager,
                               aBuilder);
       return NS_OK;
     }
@@ -726,10 +748,16 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       *mTwo.node = GetDocumentFragmentForTemplate(node);
       return NS_OK;
     }
-    case eTreeOpMarkAsBroken: {
-      aBuilder->MarkAsBroken(NS_ERROR_OUT_OF_MEMORY);
+    case eTreeOpGetFosterParent: {
+      nsIContent* table = *(mOne.node);
+      nsIContent* stackParent = *(mTwo.node);
+      nsIContent* fosterParent = GetFosterParent(table, stackParent);
+      *mThree.node = fosterParent;
       return NS_OK;
     }
+    case eTreeOpMarkAsBroken: {
+      return mOne.result;
+    }
     case eTreeOpRunScript: {
       nsIContent* node = *(mOne.node);
       nsAHtml5TreeBuilderState* snapshot = mTwo.state;
@@ -797,14 +825,6 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       aBuilder->DidBuildModel(false); // this causes a notifications flush anyway
       return NS_OK;
     }
-    case eTreeOpStartLayout: {
-      aBuilder->StartLayout(); // this causes a notification flush anyway
-      return NS_OK;
-    }
-    case eTreeOpDocumentMode: {
-      aBuilder->SetDocumentMode(mOne.mode);
-      return NS_OK;
-    }
     case eTreeOpSetStyleLineNumber: {
       nsIContent* node = *(mOne.node);
       nsCOMPtr<nsIStyleSheetLinkingElement> ssle = do_QueryInterface(node);
@@ -848,14 +868,6 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       }
       return NS_OK;
     }
-    case eTreeOpAddLineNumberId: {
-      nsIContent* node = *(mOne.node);
-      int32_t lineNumber = mFour.integer;
-      nsAutoString val(NS_LITERAL_STRING("line"));
-      val.AppendInt(lineNumber);
-      node->SetAttr(kNameSpaceID_None, nsGkAtoms::id, val, true);
-      return NS_OK;
-    }
     case eTreeOpAddViewSourceHref: {
       nsIContent* node = *mOne.node;
       char16_t* buffer = mTwo.unicharPtr;
@@ -911,6 +923,13 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       node->SetAttr(kNameSpaceID_None, nsGkAtoms::href, utf16, true);
       return rv;
     }
+    case eTreeOpAddViewSourceBase: {
+      char16_t* buffer = mTwo.unicharPtr;
+      int32_t length = mFour.integer;
+      nsDependentString baseUrl(buffer, length);
+      aBuilder->AddBase(baseUrl);
+      return NS_OK;
+    }
     case eTreeOpAddError: {
       nsIContent* node = *(mOne.node);
       char* msgId = mTwo.charPtr;
@@ -959,8 +978,20 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       }
       return rv;
     }
+    case eTreeOpAddLineNumberId: {
+      nsIContent* node = *(mOne.node);
+      int32_t lineNumber = mFour.integer;
+      nsAutoString val(NS_LITERAL_STRING("line"));
+      val.AppendInt(lineNumber);
+      node->SetAttr(kNameSpaceID_None, nsGkAtoms::id, val, true);
+      return NS_OK;
+    }
+    case eTreeOpStartLayout: {
+      aBuilder->StartLayout(); // this causes a notification flush anyway
+      return NS_OK;
+    }
     default: {
-      NS_NOTREACHED("Bogus tree op");
+      MOZ_CRASH("Bogus tree op");
     }
   }
   return NS_OK; // keep compiler happy
