@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -15,6 +16,7 @@
 #include <windows.h>
 #elif defined(MOZ_WIDGET_COCOA)
 #include <CoreServices/CoreServices.h>
+#include "nsCocoaFeatures.h"
 #elif defined(MOZ_WIDGET_GTK)
 #include <gtk/gtk.h>
 #endif
@@ -35,6 +37,18 @@
 #include "nsIScriptError.h"
 #include "nsIXULAppInfo.h"
 #include "nsIXULRuntime.h"
+#ifdef MOZ_B2G_LOADER
+#include "mozilla/XPTInterfaceInfoManager.h"
+#endif
+
+#ifdef MOZ_B2G_LOADER
+#define XPTONLY_MANIFEST &nsComponentManagerImpl::XPTOnlyManifestManifest
+#define XPTONLY_XPT &nsComponentManagerImpl::XPTOnlyManifestXPT
+#else
+#define XPTONLY_MANIFEST nullptr
+#define XPTONLY_XPT nullptr
+#endif
+
 
 using namespace mozilla;
 
@@ -43,8 +57,10 @@ struct ManifestDirective
   const char* directive;
   int argc;
 
-  // Some directives should only be delivered for NS_COMPONENT_LOCATION
-  // manifests.
+  // Binary components are only allowed for APP locations.
+  bool apponly;
+
+  // Some directives should only be delivered for APP or EXTENSION locations.
   bool componentonly;
 
   bool ischrome;
@@ -56,81 +72,123 @@ struct ManifestDirective
 
   // Function to handle this directive. This isn't a union because C++ still
   // hasn't learned how to initialize unions in a sane way.
-  void (nsComponentManagerImpl::*mgrfunc)
-    (nsComponentManagerImpl::ManifestProcessingContext& cx,
-     int lineno, char *const * argv);
-  void (nsChromeRegistry::*regfunc)
-    (nsChromeRegistry::ManifestProcessingContext& cx,
-     int lineno, char *const *argv,
-     bool platform, bool contentaccessible);
+  void (nsComponentManagerImpl::*mgrfunc)(
+    nsComponentManagerImpl::ManifestProcessingContext& aCx,
+    int aLineNo, char* const* aArgv);
+  void (nsChromeRegistry::*regfunc)(
+    nsChromeRegistry::ManifestProcessingContext& aCx,
+    int aLineNo, char* const* aArgv, int aFlags);
+#ifdef MOZ_B2G_LOADER
+  // The function to handle the directive for XPT Only parsing.
+  void (*xptonlyfunc)(
+    nsComponentManagerImpl::XPTOnlyManifestProcessingContext& aCx,
+    int aLineNo, char* const* aArgv);
+#else
+  void* xptonlyfunc;
+#endif
 
   bool isContract;
 };
 static const ManifestDirective kParsingTable[] = {
-  { "manifest",         1, false, true, true, false,
-    &nsComponentManagerImpl::ManifestManifest, nullptr },
-  { "binary-component", 1, true, false, false, false,
-    &nsComponentManagerImpl::ManifestBinaryComponent, nullptr },
-  { "interfaces",       1, true, false, false, false,
-    &nsComponentManagerImpl::ManifestXPT, nullptr },
-  { "component",        2, true, false, false, false,
-    &nsComponentManagerImpl::ManifestComponent, nullptr },
-  { "contract",         2, true, false, false, false,
-    &nsComponentManagerImpl::ManifestContract, nullptr, true},
-  { "category",         3, true, false, false, false,
-    &nsComponentManagerImpl::ManifestCategory, nullptr },
-  { "content",          2, true, true, true,  true,
-    nullptr, &nsChromeRegistry::ManifestContent },
-  { "locale",           3, true, true, true,  false,
-    nullptr, &nsChromeRegistry::ManifestLocale },
-  { "skin",             3, false, true, true,  false,
-    nullptr, &nsChromeRegistry::ManifestSkin },
-  { "overlay",          2, true, true, false,  false,
-    nullptr, &nsChromeRegistry::ManifestOverlay },
-  { "style",            2, false, true, false,  false,
-    nullptr, &nsChromeRegistry::ManifestStyle },
-  { "override",         2, true, true, true,  false,
-    nullptr, &nsChromeRegistry::ManifestOverride },
-  { "resource",         2, true, true, false,  false,
-    nullptr, &nsChromeRegistry::ManifestResource }
+  {
+    "manifest",         1, false, false, true, true, false,
+    &nsComponentManagerImpl::ManifestManifest, nullptr, XPTONLY_MANIFEST
+  },
+  {
+    "binary-component", 1, true, true, false, false, false,
+    &nsComponentManagerImpl::ManifestBinaryComponent, nullptr, nullptr
+  },
+  {
+    "interfaces",       1, false, true, false, false, false,
+    &nsComponentManagerImpl::ManifestXPT, nullptr, XPTONLY_XPT
+  },
+  {
+    "component",        2, false, true, false, false, false,
+    &nsComponentManagerImpl::ManifestComponent, nullptr, nullptr
+  },
+  {
+    "contract",         2, false, true, false, false, false,
+    &nsComponentManagerImpl::ManifestContract, nullptr, nullptr, true
+  },
+  {
+    "category",         3, false, true, false, false, false,
+    &nsComponentManagerImpl::ManifestCategory, nullptr, nullptr
+  },
+  {
+    "content",          2, false, true, true, true,  true,
+    nullptr, &nsChromeRegistry::ManifestContent, nullptr
+  },
+  {
+    "locale",           3, false, true, true, true, false,
+    nullptr, &nsChromeRegistry::ManifestLocale, nullptr
+  },
+  {
+    "skin",             3, false, false, true, true, false,
+    nullptr, &nsChromeRegistry::ManifestSkin, nullptr
+  },
+  {
+    "overlay",          2, false, true, true, false, false,
+    nullptr, &nsChromeRegistry::ManifestOverlay, nullptr
+  },
+  {
+    "style",            2, false, false, true, false, false,
+    nullptr, &nsChromeRegistry::ManifestStyle, nullptr
+  },
+  {
+    // NB: note that while skin manifests can use this, they are only allowed
+    // to use it for chrome://../skin/ URLs
+    "override",         2, false, false, true, true, false,
+    nullptr, &nsChromeRegistry::ManifestOverride, nullptr
+  },
+  {
+    "resource",         2, false, true, true, true, false,
+    nullptr, &nsChromeRegistry::ManifestResource, nullptr
+  }
 };
 
 static const char kWhitespace[] = "\t ";
 
-static bool IsNewline(char c)
+static bool
+IsNewline(char aChar)
 {
-  return c == '\n' || c == '\r';
+  return aChar == '\n' || aChar == '\r';
 }
 
 namespace {
 struct AutoPR_smprintf_free
 {
-  AutoPR_smprintf_free(char* buf)
-    : mBuf(buf)
-  {
-  }
+  explicit AutoPR_smprintf_free(char* aBuf) : mBuf(aBuf) {}
 
   ~AutoPR_smprintf_free()
   {
-    if (mBuf)
+    if (mBuf) {
       PR_smprintf_free(mBuf);
+    }
   }
 
-  operator char*() const {
-    return mBuf;
-  }
+  operator char*() const { return mBuf; }
 
   char* mBuf;
 };
 
-} // anonymous namespace
+} // namespace
 
-void LogMessage(const char* aMsg, ...)
+/**
+ * If we are pre-loading XPTs, this method may do nothing because the
+ * console service is not initialized.
+ */
+void
+LogMessage(const char* aMsg, ...)
 {
+  if (!nsComponentManagerImpl::gComponentManager) {
+    return;
+  }
+
   nsCOMPtr<nsIConsoleService> console =
     do_GetService(NS_CONSOLESERVICE_CONTRACTID);
-  if (!console)
+  if (!console) {
     return;
+  }
 
   va_list args;
   va_start(args, aMsg);
@@ -142,15 +200,25 @@ void LogMessage(const char* aMsg, ...)
   console->LogMessage(error);
 }
 
-void LogMessageWithContext(FileLocation &aFile,
-                           uint32_t aLineNumber, const char* aMsg, ...)
+/**
+ * If we are pre-loading XPTs, this method may do nothing because the
+ * console service is not initialized.
+ */
+void
+LogMessageWithContext(FileLocation& aFile,
+                      uint32_t aLineNumber, const char* aMsg, ...)
 {
   va_list args;
   va_start(args, aMsg);
   AutoPR_smprintf_free formatted(PR_vsmprintf(aMsg, args));
   va_end(args);
-  if (!formatted)
+  if (!formatted) {
     return;
+  }
+
+  if (!nsComponentManagerImpl::gComponentManager) {
+    return;
+  }
 
   nsCString file;
   aFile.GetURIString(file);
@@ -161,21 +229,23 @@ void LogMessageWithContext(FileLocation &aFile,
     // This can happen early in component registration. Fall back to a
     // generic console message.
     LogMessage("Warning: in '%s', line %i: %s", file.get(),
-               aLineNumber, (char*) formatted);
+               aLineNumber, (char*)formatted);
     return;
   }
 
   nsCOMPtr<nsIConsoleService> console =
     do_GetService(NS_CONSOLESERVICE_CONTRACTID);
-  if (!console)
+  if (!console) {
     return;
+  }
 
   nsresult rv = error->Init(NS_ConvertUTF8toUTF16(formatted),
-			    NS_ConvertUTF8toUTF16(file), EmptyString(),
-			    aLineNumber, 0, nsIScriptError::warningFlag,
-			    "chrome registration");
-  if (NS_FAILED(rv))
+                            NS_ConvertUTF8toUTF16(file), EmptyString(),
+                            aLineNumber, 0, nsIScriptError::warningFlag,
+                            "chrome registration");
+  if (NS_FAILED(rv)) {
     return;
+  }
 
   console->LogMessage(error);
 }
@@ -194,8 +264,9 @@ void LogMessageWithContext(FileLocation &aFile,
 static bool
 CheckFlag(const nsSubstring& aFlag, const nsSubstring& aData, bool& aResult)
 {
-  if (!StringBeginsWith(aData, aFlag))
+  if (!StringBeginsWith(aData, aFlag)) {
     return false;
+  }
 
   if (aFlag.Length() == aData.Length()) {
     // the data is simply "flag", which is the same as "flag=yes"
@@ -214,23 +285,24 @@ CheckFlag(const nsSubstring& aFlag, const nsSubstring& aData, bool& aResult)
   }
 
   switch (aData.CharAt(aFlag.Length() + 1)) {
-  case '1':
-  case 't': //true
-  case 'y': //yes
-    aResult = true;
-    return true;
+    case '1':
+    case 't': //true
+    case 'y': //yes
+      aResult = true;
+      return true;
 
-  case '0':
-  case 'f': //false
-  case 'n': //no
-    aResult = false;
-    return true;
+    case '0':
+    case 'f': //false
+    case 'n': //no
+      aResult = false;
+      return true;
   }
 
   return false;
 }
 
-enum TriState {
+enum TriState
+{
   eUnspecified,
   eBad,
   eOK
@@ -252,28 +324,33 @@ static bool
 CheckStringFlag(const nsSubstring& aFlag, const nsSubstring& aData,
                 const nsSubstring& aValue, TriState& aResult)
 {
-  if (aData.Length() < aFlag.Length() + 1)
+  if (aData.Length() < aFlag.Length() + 1) {
     return false;
+  }
 
-  if (!StringBeginsWith(aData, aFlag))
+  if (!StringBeginsWith(aData, aFlag)) {
     return false;
+  }
 
   bool comparison = true;
   if (aData[aFlag.Length()] != '=') {
     if (aData[aFlag.Length()] == '!' &&
         aData.Length() >= aFlag.Length() + 2 &&
-        aData[aFlag.Length() + 1] == '=')
+        aData[aFlag.Length() + 1] == '=') {
       comparison = false;
-    else
+    } else {
       return false;
+    }
   }
 
   if (aResult != eOK) {
-    nsDependentSubstring testdata = Substring(aData, aFlag.Length() + (comparison ? 1 : 2));
-    if (testdata.Equals(aValue))
+    nsDependentSubstring testdata =
+      Substring(aData, aFlag.Length() + (comparison ? 1 : 2));
+    if (testdata.Equals(aValue)) {
       aResult = comparison ? eOK : eBad;
-    else
+    } else {
       aResult = comparison ? eBad : eOK;
+    }
   }
 
   return true;
@@ -304,15 +381,18 @@ static bool
 CheckVersionFlag(const nsString& aFlag, const nsString& aData,
                  const nsString& aValue, TriState& aResult)
 {
-  if (aData.Length() < aFlag.Length() + 2)
+  if (aData.Length() < aFlag.Length() + 2) {
     return false;
+  }
 
-  if (!StringBeginsWith(aData, aFlag))
+  if (!StringBeginsWith(aData, aFlag)) {
     return false;
+  }
 
   if (aValue.Length() == 0) {
-    if (aResult != eOK)
+    if (aResult != eOK) {
       aResult = eBad;
+    }
     return true;
   }
 
@@ -320,49 +400,49 @@ CheckVersionFlag(const nsString& aFlag, const nsString& aData,
   nsAutoString testdata;
 
   switch (aData[aFlag.Length()]) {
-  case '=':
-    comparison = COMPARE_EQ;
-    testdata = Substring(aData, aFlag.Length() + 1);
-    break;
-
-  case '<':
-    if (aData[aFlag.Length() + 1] == '=') {
-      comparison = COMPARE_EQ | COMPARE_LT;
-      testdata = Substring(aData, aFlag.Length() + 2);
-    }
-    else {
-      comparison = COMPARE_LT;
+    case '=':
+      comparison = COMPARE_EQ;
       testdata = Substring(aData, aFlag.Length() + 1);
-    }
-    break;
+      break;
 
-  case '>':
-    if (aData[aFlag.Length() + 1] == '=') {
-      comparison = COMPARE_EQ | COMPARE_GT;
-      testdata = Substring(aData, aFlag.Length() + 2);
-    }
-    else {
-      comparison = COMPARE_GT;
-      testdata = Substring(aData, aFlag.Length() + 1);
-    }
-    break;
+    case '<':
+      if (aData[aFlag.Length() + 1] == '=') {
+        comparison = COMPARE_EQ | COMPARE_LT;
+        testdata = Substring(aData, aFlag.Length() + 2);
+      } else {
+        comparison = COMPARE_LT;
+        testdata = Substring(aData, aFlag.Length() + 1);
+      }
+      break;
 
-  default:
-    return false;
+    case '>':
+      if (aData[aFlag.Length() + 1] == '=') {
+        comparison = COMPARE_EQ | COMPARE_GT;
+        testdata = Substring(aData, aFlag.Length() + 2);
+      } else {
+        comparison = COMPARE_GT;
+        testdata = Substring(aData, aFlag.Length() + 1);
+      }
+      break;
+
+    default:
+      return false;
   }
 
-  if (testdata.Length() == 0)
+  if (testdata.Length() == 0) {
     return false;
+  }
 
   if (aResult != eOK) {
     int32_t c = mozilla::CompareVersions(NS_ConvertUTF16toUTF8(aValue).get(),
                                          NS_ConvertUTF16toUTF8(testdata).get());
     if ((c == 0 && comparison & COMPARE_EQ) ||
-	(c < 0 && comparison & COMPARE_LT) ||
-	(c > 0 && comparison & COMPARE_GT))
+        (c < 0 && comparison & COMPARE_LT) ||
+        (c > 0 && comparison & COMPARE_GT)) {
       aResult = eOK;
-    else
+    } else {
       aResult = eBad;
+    }
   }
 
   return true;
@@ -370,10 +450,11 @@ CheckVersionFlag(const nsString& aFlag, const nsString& aData,
 
 // In-place conversion of ascii characters to lower case
 static void
-ToLowerCase(char* token)
+ToLowerCase(char* aToken)
 {
-  for (; *token; ++token)
-    *token = NS_ToLower(*token);
+  for (; *aToken; ++aToken) {
+    *aToken = NS_ToLower(*aToken);
+  }
 }
 
 namespace {
@@ -384,28 +465,48 @@ struct CachedDirective
   char* argv[4];
 };
 
-} // anonymous namespace
+} // namespace
 
 
+/**
+ * For XPT-Only mode, the parser handles only directives of "manifest"
+ * and "interfaces", and always call the function given by |xptonlyfunc|
+ * variable of struct |ManifestDirective|.
+ *
+ * This function is safe to be called before the component manager is
+ * ready if aXPTOnly is true for it don't invoke any component during
+ * parsing.
+ */
 void
-ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOnly)
+ParseManifest(NSLocationType aType, FileLocation& aFile, char* aBuf,
+              bool aChromeOnly, bool aXPTOnly)
 {
-  nsComponentManagerImpl::ManifestProcessingContext mgrcx(type, file, aChromeOnly);
-  nsChromeRegistry::ManifestProcessingContext chromecx(type, file);
+  nsComponentManagerImpl::ManifestProcessingContext mgrcx(aType, aFile,
+                                                          aChromeOnly);
+  nsChromeRegistry::ManifestProcessingContext chromecx(aType, aFile);
+#ifdef MOZ_B2G_LOADER
+  nsComponentManagerImpl::XPTOnlyManifestProcessingContext xptonlycx(aFile);
+#endif
   nsresult rv;
 
   NS_NAMED_LITERAL_STRING(kPlatform, "platform");
   NS_NAMED_LITERAL_STRING(kContentAccessible, "contentaccessible");
+  NS_NAMED_LITERAL_STRING(kRemoteEnabled, "remoteenabled");
+  NS_NAMED_LITERAL_STRING(kRemoteRequired, "remoterequired");
   NS_NAMED_LITERAL_STRING(kApplication, "application");
   NS_NAMED_LITERAL_STRING(kAppVersion, "appversion");
   NS_NAMED_LITERAL_STRING(kGeckoVersion, "platformversion");
   NS_NAMED_LITERAL_STRING(kOs, "os");
   NS_NAMED_LITERAL_STRING(kOsVersion, "osversion");
   NS_NAMED_LITERAL_STRING(kABI, "abi");
+  NS_NAMED_LITERAL_STRING(kProcess, "process");
 #if defined(MOZ_WIDGET_ANDROID)
   NS_NAMED_LITERAL_STRING(kTablet, "tablet");
 #endif
 
+  NS_NAMED_LITERAL_STRING(kMain, "main");
+  NS_NAMED_LITERAL_STRING(kContent, "content");
+
   // Obsolete
   NS_NAMED_LITERAL_STRING(kXPCNativeWrappers, "xpcnativewrappers");
 
@@ -414,23 +515,32 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
   nsAutoString geckoVersion;
   nsAutoString osTarget;
   nsAutoString abi;
+  nsAutoString process;
 
-  nsCOMPtr<nsIXULAppInfo> xapp (do_GetService(XULAPPINFO_SERVICE_CONTRACTID));
+  nsCOMPtr<nsIXULAppInfo> xapp;
+  if (!aXPTOnly) {
+    // Avoid to create any component for XPT only mode.
+    // No xapp means no ID, version, ..., modifiers checking.
+    xapp = do_GetService(XULAPPINFO_SERVICE_CONTRACTID);
+  }
   if (xapp) {
     nsAutoCString s;
     rv = xapp->GetID(s);
-    if (NS_SUCCEEDED(rv))
+    if (NS_SUCCEEDED(rv)) {
       CopyUTF8toUTF16(s, appID);
+    }
 
     rv = xapp->GetVersion(s);
-    if (NS_SUCCEEDED(rv))
+    if (NS_SUCCEEDED(rv)) {
       CopyUTF8toUTF16(s, appVersion);
+    }
 
     rv = xapp->GetPlatformVersion(s);
-    if (NS_SUCCEEDED(rv))
+    if (NS_SUCCEEDED(rv)) {
       CopyUTF8toUTF16(s, geckoVersion);
+    }
 
-    nsCOMPtr<nsIXULRuntime> xruntime (do_QueryInterface(xapp));
+    nsCOMPtr<nsIXULRuntime> xruntime(do_QueryInterface(xapp));
     if (xruntime) {
       rv = xruntime->GetOS(s);
       if (NS_SUCCEEDED(rv)) {
@@ -442,7 +552,7 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
       if (NS_SUCCEEDED(rv) && osTarget.Length()) {
         ToLowerCase(s);
         CopyUTF8toUTF16(s, abi);
-        abi.Insert(PRUnichar('_'), 0);
+        abi.Insert(char16_t('_'), 0);
         abi.Insert(osTarget, 0);
       }
     }
@@ -450,38 +560,47 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
 
   nsAutoString osVersion;
 #if defined(XP_WIN)
+#pragma warning(push)
+#pragma warning(disable:4996) // VC12+ deprecates GetVersionEx
   OSVERSIONINFO info = { sizeof(OSVERSIONINFO) };
   if (GetVersionEx(&info)) {
-    nsTextFormatter::ssprintf(osVersion, NS_LITERAL_STRING("%ld.%ld").get(),
-                                         info.dwMajorVersion,
-                                         info.dwMinorVersion);
+    nsTextFormatter::ssprintf(osVersion, u"%ld.%ld",
+                              info.dwMajorVersion,
+                              info.dwMinorVersion);
   }
+#pragma warning(pop)
 #elif defined(MOZ_WIDGET_COCOA)
-  SInt32 majorVersion, minorVersion;
-  if ((Gestalt(gestaltSystemVersionMajor, &majorVersion) == noErr) &&
-      (Gestalt(gestaltSystemVersionMinor, &minorVersion) == noErr)) {
-    nsTextFormatter::ssprintf(osVersion, NS_LITERAL_STRING("%ld.%ld").get(),
-                                         majorVersion,
-                                         minorVersion);
-  }
+  SInt32 majorVersion = nsCocoaFeatures::OSXVersionMajor();
+  SInt32 minorVersion = nsCocoaFeatures::OSXVersionMinor();
+  nsTextFormatter::ssprintf(osVersion, u"%ld.%ld",
+                            majorVersion,
+                            minorVersion);
 #elif defined(MOZ_WIDGET_GTK)
-  nsTextFormatter::ssprintf(osVersion, NS_LITERAL_STRING("%ld.%ld").get(),
-                                       gtk_major_version,
-                                       gtk_minor_version);
+  nsTextFormatter::ssprintf(osVersion, u"%ld.%ld",
+                            gtk_major_version,
+                            gtk_minor_version);
 #elif defined(MOZ_WIDGET_ANDROID)
   bool isTablet = false;
   if (mozilla::AndroidBridge::Bridge()) {
-    mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build$VERSION", "RELEASE", osVersion);
-    isTablet = mozilla::widget::android::GeckoAppShell::IsTablet();
+    mozilla::AndroidBridge::Bridge()->GetStaticStringField("android/os/Build$VERSION",
+                                                           "RELEASE",
+                                                           osVersion);
+    isTablet = java::GeckoAppShell::IsTablet();
   }
 #endif
 
+  if (XRE_IsContentProcess()) {
+    process = kContent;
+  } else {
+    process = kMain;
+  }
+
   // Because contracts must be registered after CIDs, we save and process them
   // at the end.
   nsTArray<CachedDirective> contracts;
 
-  char *token;
-  char *newline = buf;
+  char* token;
+  char* newline = aBuf;
   uint32_t line = 0;
 
   // outer loop tokenizes by newline
@@ -490,12 +609,14 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
       ++newline;
       ++line;
     }
-    if (!*newline)
+    if (!*newline) {
       break;
+    }
 
     token = newline;
-    while (*newline && !IsNewline(*newline))
+    while (*newline && !IsNewline(*newline)) {
       ++newline;
+    }
 
     if (*newline) {
       *newline = '\0';
@@ -503,39 +624,51 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
     }
     ++line;
 
-    if (*token == '#') // ignore lines that begin with # as comments
+    if (*token == '#') { // ignore lines that begin with # as comments
       continue;
+    }
 
-    char *whitespace = token;
+    char* whitespace = token;
     token = nsCRT::strtok(whitespace, kWhitespace, &whitespace);
-    if (!token) continue;
+    if (!token) {
+      continue;
+    }
 
     const ManifestDirective* directive = nullptr;
     for (const ManifestDirective* d = kParsingTable;
-	 d < ArrayEnd(kParsingTable);
-	 ++d) {
-      if (!strcmp(d->directive, token)) {
-	directive = d;
-	break;
+         d < ArrayEnd(kParsingTable);
+         ++d) {
+      if (!strcmp(d->directive, token) &&
+          (!aXPTOnly || d->xptonlyfunc)) {
+        directive = d;
+        break;
       }
     }
 
     if (!directive) {
-      LogMessageWithContext(file, line,
+      LogMessageWithContext(aFile, line,
                             "Ignoring unrecognized chrome manifest directive '%s'.",
                             token);
       continue;
     }
 
-    if (!directive->allowbootstrap && NS_BOOTSTRAPPED_LOCATION == type) {
-      LogMessageWithContext(file, line,
+    if (!directive->allowbootstrap && NS_BOOTSTRAPPED_LOCATION == aType) {
+      LogMessageWithContext(aFile, line,
                             "Bootstrapped manifest not allowed to use '%s' directive.",
                             token);
       continue;
     }
 
-    if (directive->componentonly && NS_SKIN_LOCATION == type) {
-      LogMessageWithContext(file, line,
+#ifndef MOZ_BINARY_EXTENSIONS
+    if (directive->apponly && NS_APP_LOCATION != aType) {
+      LogMessageWithContext(aFile, line,
+                            "Only application manifests may use the '%s' directive.", token);
+      continue;
+    }
+#endif
+
+    if (directive->componentonly && NS_SKIN_LOCATION == aType) {
+      LogMessageWithContext(aFile, line,
                             "Skin manifest not allowed to use '%s' directive.",
                             token);
       continue;
@@ -543,11 +676,12 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
 
     NS_ASSERTION(directive->argc < 4, "Need to reset argv array length");
     char* argv[4];
-    for (int i = 0; i < directive->argc; ++i)
+    for (int i = 0; i < directive->argc; ++i) {
       argv[i] = nsCRT::strtok(whitespace, kWhitespace, &whitespace);
+    }
 
     if (!argv[directive->argc - 1]) {
-      LogMessageWithContext(file, line,
+      LogMessageWithContext(aFile, line,
                             "Not enough arguments for chrome manifest directive '%s', expected %i.",
                             token, directive->argc);
       continue;
@@ -560,23 +694,26 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
     TriState stOsVersion = eUnspecified;
     TriState stOs = eUnspecified;
     TriState stABI = eUnspecified;
+    TriState stProcess = eUnspecified;
 #if defined(MOZ_WIDGET_ANDROID)
     TriState stTablet = eUnspecified;
 #endif
-    bool platform = false;
-    bool contentAccessible = false;
+    int flags = 0;
 
-    while (nullptr != (token = nsCRT::strtok(whitespace, kWhitespace, &whitespace)) && ok) {
+    while ((token = nsCRT::strtok(whitespace, kWhitespace, &whitespace)) &&
+           ok) {
       ToLowerCase(token);
       NS_ConvertASCIItoUTF16 wtoken(token);
 
       if (CheckStringFlag(kApplication, wtoken, appID, stApp) ||
           CheckStringFlag(kOs, wtoken, osTarget, stOs) ||
           CheckStringFlag(kABI, wtoken, abi, stABI) ||
+          CheckStringFlag(kProcess, wtoken, process, stProcess) ||
           CheckVersionFlag(kOsVersion, wtoken, osVersion, stOsVersion) ||
           CheckVersionFlag(kAppVersion, wtoken, appVersion, stAppVersion) ||
-          CheckVersionFlag(kGeckoVersion, wtoken, geckoVersion, stGeckoVersion))
+          CheckVersionFlag(kGeckoVersion, wtoken, geckoVersion, stGeckoVersion)) {
         continue;
+      }
 
 #if defined(MOZ_WIDGET_ANDROID)
       bool tablet = false;
@@ -586,20 +723,39 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
       }
 #endif
 
-      if (directive->contentflags &&
-          (CheckFlag(kPlatform, wtoken, platform) ||
-           CheckFlag(kContentAccessible, wtoken, contentAccessible)))
-        continue;
+      if (directive->contentflags) {
+        bool flag;
+        if (CheckFlag(kPlatform, wtoken, flag)) {
+          if (flag)
+            flags |= nsChromeRegistry::PLATFORM_PACKAGE;
+          continue;
+        }
+        if (CheckFlag(kContentAccessible, wtoken, flag)) {
+          if (flag)
+            flags |= nsChromeRegistry::CONTENT_ACCESSIBLE;
+          continue;
+        }
+        if (CheckFlag(kRemoteEnabled, wtoken, flag)) {
+          if (flag)
+            flags |= nsChromeRegistry::REMOTE_ALLOWED;
+          continue;
+        }
+        if (CheckFlag(kRemoteRequired, wtoken, flag)) {
+          if (flag)
+            flags |= nsChromeRegistry::REMOTE_REQUIRED;
+          continue;
+        }
+      }
 
       bool xpcNativeWrappers = true; // Dummy for CheckFlag.
       if (CheckFlag(kXPCNativeWrappers, wtoken, xpcNativeWrappers)) {
-        LogMessageWithContext(file, line,
+        LogMessageWithContext(aFile, line,
                               "Ignoring obsolete chrome registration modifier '%s'.",
                               token);
         continue;
       }
 
-      LogMessageWithContext(file, line,
+      LogMessageWithContext(aFile, line,
                             "Unrecognized chrome manifest modifier '%s'.",
                             token);
       ok = false;
@@ -614,42 +770,50 @@ ParseManifest(NSLocationType type, FileLocation &file, char* buf, bool aChromeOn
 #ifdef MOZ_WIDGET_ANDROID
         stTablet == eBad ||
 #endif
-        stABI == eBad)
+        stABI == eBad ||
+        stProcess == eBad) {
       continue;
+    }
 
+#ifdef MOZ_B2G_LOADER
+    if (aXPTOnly) {
+      directive->xptonlyfunc(xptonlycx, line, argv);
+    } else
+#endif /* MOZ_B2G_LOADER */
     if (directive->regfunc) {
-      if (GeckoProcessType_Default != XRE_GetProcessType())
+      if (GeckoProcessType_Default != XRE_GetProcessType()) {
         continue;
+      }
 
       if (!nsChromeRegistry::gChromeRegistry) {
         nsCOMPtr<nsIChromeRegistry> cr =
           mozilla::services::GetChromeRegistryService();
         if (!nsChromeRegistry::gChromeRegistry) {
-          LogMessageWithContext(file, line,
+          LogMessageWithContext(aFile, line,
                                 "Chrome registry isn't available yet.");
           continue;
         }
       }
 
-      (nsChromeRegistry::gChromeRegistry->*(directive->regfunc))
-	(chromecx, line, argv, platform, contentAccessible);
-    }
-    else if (directive->ischrome || !aChromeOnly) {
+      (nsChromeRegistry::gChromeRegistry->*(directive->regfunc))(
+        chromecx, line, argv, flags);
+    } else if (directive->ischrome || !aChromeOnly) {
       if (directive->isContract) {
         CachedDirective* cd = contracts.AppendElement();
         cd->lineno = line;
         cd->argv[0] = argv[0];
         cd->argv[1] = argv[1];
+      } else {
+        (nsComponentManagerImpl::gComponentManager->*(directive->mgrfunc))(
+          mgrcx, line, argv);
       }
-      else
-        (nsComponentManagerImpl::gComponentManager->*(directive->mgrfunc))
-          (mgrcx, line, argv);
     }
   }
 
   for (uint32_t i = 0; i < contracts.Length(); ++i) {
     CachedDirective& d = contracts[i];
-    nsComponentManagerImpl::gComponentManager->ManifestContract
-      (mgrcx, d.lineno, d.argv);
+    nsComponentManagerImpl::gComponentManager->ManifestContract(mgrcx,
+                                                                d.lineno,
+                                                                d.argv);
   }
 }
